@@ -23,8 +23,8 @@ macro_rules! _unroll_one_eq_16 {
         let aa_ptr = unsafe { $a_ptr.add($loop_size * $idx) };
         let bb_ptr = unsafe { $b_ptr.add($loop_size * $idx) };
         //
-        let aac = unsafe { *(aa_ptr as *const u128) };
-        let bbc = unsafe { *(bb_ptr as *const u128) };
+        let aac = unsafe { crate::utils::_read_a_native_endian_u128(std::slice::from_raw_parts(aa_ptr, 16)) };
+        let bbc = unsafe { crate::utils::_read_a_native_endian_u128(std::slice::from_raw_parts(bb_ptr, 16)) };
         if aac != bbc {
             return false;
         }
@@ -36,8 +36,8 @@ macro_rules! _unroll_one_eq_8 {
         let aa_ptr = unsafe { $a_ptr.add($loop_size * $idx) };
         let bb_ptr = unsafe { $b_ptr.add($loop_size * $idx) };
         //
-        let aac = unsafe { *(aa_ptr as *const u64) };
-        let bbc = unsafe { *(bb_ptr as *const u64) };
+        let aac = unsafe { crate::utils::_read_a_native_endian_u64(std::slice::from_raw_parts(aa_ptr, 8)) };
+        let bbc = unsafe { crate::utils::_read_a_native_endian_u64(std::slice::from_raw_parts(bb_ptr, 8)) };
         if aac != bbc {
             return false;
         }
@@ -49,8 +49,8 @@ macro_rules! _unroll_one_eq_4 {
         let aa_ptr = unsafe { $a_ptr.add($loop_size * $idx) };
         let bb_ptr = unsafe { $b_ptr.add($loop_size * $idx) };
         //
-        let aac = unsafe { *(aa_ptr as *const u32) };
-        let bbc = unsafe { *(bb_ptr as *const u32) };
+        let aac = unsafe { crate::utils::_read_a_native_endian_u32(std::slice::from_raw_parts(aa_ptr, 4)) };
+        let bbc = unsafe { crate::utils::_read_a_native_endian_u32(std::slice::from_raw_parts(bb_ptr, 4)) };
         if aac != bbc {
             return false;
         }
@@ -62,8 +62,8 @@ macro_rules! _unroll_one_eq_2 {
         let aa_ptr = unsafe { $a_ptr.add($loop_size * $idx) };
         let bb_ptr = unsafe { $b_ptr.add($loop_size * $idx) };
         //
-        let aac = unsafe { *(aa_ptr as *const u16) };
-        let bbc = unsafe { *(bb_ptr as *const u16) };
+        let aac = unsafe { crate::utils::_read_a_native_endian_u16(std::slice::from_raw_parts(aa_ptr, 2)) };
+        let bbc = unsafe { crate::utils::_read_a_native_endian_u16(std::slice::from_raw_parts(bb_ptr, 2)) };
         if aac != bbc {
             return false;
         }
@@ -99,8 +99,7 @@ fn _start_eq_128(a: &[u8], b: &[u8]) -> bool {
     {
         let unroll = 8;
         let loop_size = 16;
-        let end_ptr_16_8 = unsafe { end_ptr.sub(loop_size * unroll) };
-        while a_ptr <= end_ptr_16_8 {
+        while unsafe { end_ptr.offset_from(a_ptr) } >= (loop_size * unroll) as isize {
             _unroll_one_eq_16!(a_ptr, b_ptr, loop_size, 0);
             _unroll_one_eq_16!(a_ptr, b_ptr, loop_size, 1);
             _unroll_one_eq_16!(a_ptr, b_ptr, loop_size, 2);
@@ -116,8 +115,7 @@ fn _start_eq_128(a: &[u8], b: &[u8]) -> bool {
     }
     {
         let loop_size = 16;
-        let end_ptr_16 = unsafe { end_ptr.sub(loop_size) };
-        while a_ptr <= end_ptr_16 {
+        while unsafe { end_ptr.offset_from(a_ptr) } >= loop_size as isize {
             _unroll_one_eq_16!(a_ptr, b_ptr, loop_size, 0);
             a_ptr = unsafe { a_ptr.add(loop_size) };
             b_ptr = unsafe { b_ptr.add(loop_size) };
@@ -143,8 +141,7 @@ fn _start_eq_64(a: &[u8], b: &[u8]) -> bool {
     {
         let unroll = 8;
         let loop_size = 8;
-        let end_ptr_8_8 = unsafe { end_ptr.sub(loop_size * unroll) };
-        while a_ptr <= end_ptr_8_8 {
+        while unsafe { end_ptr.offset_from(a_ptr) } >= (loop_size * unroll) as isize {
             _unroll_one_eq_8!(a_ptr, b_ptr, loop_size, 0);
             _unroll_one_eq_8!(a_ptr, b_ptr, loop_size, 1);
             _unroll_one_eq_8!(a_ptr, b_ptr, loop_size, 2);
@@ -160,8 +157,7 @@ fn _start_eq_64(a: &[u8], b: &[u8]) -> bool {
     }
     {
         let loop_size = 8;
-        let end_ptr_8 = unsafe { end_ptr.sub(loop_size) };
-        while a_ptr <= end_ptr_8 {
+        while unsafe { end_ptr.offset_from(a_ptr) } >= loop_size as isize {
             _unroll_one_eq_8!(a_ptr, b_ptr, loop_size, 0);
             a_ptr = unsafe { a_ptr.add(loop_size) };
             b_ptr = unsafe { b_ptr.add(loop_size) };
@@ -187,8 +183,7 @@ fn _start_eq_32(a: &[u8], b: &[u8]) -> bool {
     {
         let unroll = 8;
         let loop_size = 4;
-        let end_ptr_4_8 = unsafe { end_ptr.sub(loop_size * unroll) };
-        while a_ptr <= end_ptr_4_8 {
+        while unsafe { end_ptr.offset_from(a_ptr) } >= (loop_size * unroll) as isize {
             _unroll_one_eq_4!(a_ptr, b_ptr, loop_size, 0);
             _unroll_one_eq_4!(a_ptr, b_ptr, loop_size, 1);
             _unroll_one_eq_4!(a_ptr, b_ptr, loop_size, 2);
@@ -204,8 +199,7 @@ fn _start_eq_32(a: &[u8], b: &[u8]) -> bool {
     }
     {
         let loop_size = 4;
-        let end_ptr_4 = unsafe { end_ptr.sub(loop_size) };
-        while a_ptr <= end_ptr_4 {
+        while unsafe { end_ptr.offset_from(a_ptr) } >= loop_size as isize {
             _unroll_one_eq_4!(a_ptr, b_ptr, loop_size, 0);
             //
             a_ptr = unsafe { a_ptr.add(loop_size) };
@@ -222,8 +216,7 @@ fn _memeq_remaining_15_bytes_impl(a_ptr: *const u8, b_ptr: *const u8, end_ptr: *
     let mut b_ptr = b_ptr;
     {
         let loop_size = 8;
-        let end_ptr_8 = unsafe { end_ptr.sub(loop_size) };
-        if a_ptr <= end_ptr_8 {
+        if unsafe { end_ptr.offset_from(a_ptr) } >= loop_size as isize {
             _unroll_one_eq_8!(a_ptr, b_ptr, loop_size, 0);
             //
             a_ptr = unsafe { a_ptr.add(loop_size) };
@@ -240,8 +233,7 @@ fn _memeq_remaining_7_bytes_impl(a_ptr: *const u8, b_ptr: *const u8, end_ptr: *c
     let mut b_ptr = b_ptr;
     {
         let loop_size = 4;
-        let end_ptr_4 = unsafe { end_ptr.sub(loop_size) };
-        if a_ptr <= end_ptr_4 {
+        if unsafe { end_ptr.offset_from(a_ptr) } >= loop_size as isize {
             _unroll_one_eq_4!(a_ptr, b_ptr, loop_size, 0);
             //
             a_ptr = unsafe { a_ptr.add(loop_size) };
@@ -258,8 +250,7 @@ fn _memeq_remaining_3_bytes_impl(a_ptr: *const u8, b_ptr: *const u8, end_ptr: *c
     let mut b_ptr = b_ptr;
     {
         let loop_size = 2;
-        let end_ptr_2 = unsafe { end_ptr.sub(loop_size) };
-        if a_ptr <= end_ptr_2 {
+        if unsafe { end_ptr.offset_from(a_ptr) } >= loop_size as isize {
             _unroll_one_eq_2!(a_ptr, b_ptr, loop_size, 0);
             //
             a_ptr = unsafe { a_ptr.add(loop_size) };
@@ -268,8 +259,7 @@ fn _memeq_remaining_3_bytes_impl(a_ptr: *const u8, b_ptr: *const u8, end_ptr: *c
     }
     {
         let loop_size = 1;
-        let end_ptr_1 = unsafe { end_ptr.sub(loop_size) };
-        if a_ptr <= end_ptr_1 {
+        if unsafe { end_ptr.offset_from(a_ptr) } >= loop_size as isize {
             _unroll_one_eq_1!(a_ptr, b_ptr, loop_size, 0);
         }
     }
